/******************************************************************************
 * Copyright (c) 2014, AllSeen Alliance. All rights reserved.
 *
 *    Permission to use, copy, modify, and/or distribute this software for any
 *    purpose with or without fee is hereby granted, provided that the above
 *    copyright notice and this permission notice appear in all copies.
 *
 *    THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 *    WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 *    MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 *    ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 *    WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 *    ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 *    OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 ******************************************************************************/

#import <Foundation/Foundation.h>
#import "AJGWCConnectorAppStatus.h"

/**
 * Implement this interface to be notified about changes in the Connector App status
 */
@protocol AJGWCConnectorAppStatusSignalHandler <NSObject>


/**
 * The event is emitted when the status of the Connector App
 * changes. Avoid blocking the thread on which the method is called.
 * @param appId The application id
 * @param status {@link AJGWCConnectorAppStatus}
 */
- (void)onStatusSignal:(NSString*) appId status:(AJGWCConnectorAppStatus*) status;

/**
 * An event could not be emitted because of an error creating its data
 * @param appId The application id
 * @param code {@link ConnectorAppStatus}
 */
- (void)onError:(NSString*) appId errorCode:(QStatus)code;

@end

